@@ -79,7 +79,7 @@ class SourceProgram:
         available_macros (tuple[str,...]):
             known macros that can be defined to affect the program's compilation (this
             is not exhaustive, additional, e.g., system macros, might be available)
-        available_macros (tuple[str,...]):
+        defined_macros (tuple[str,...]):
             macros that will be defined when compiling this program
         include_paths (tuple[str,...]):
             include paths which will be passed to the compiler (with -I)
@@ -97,6 +97,23 @@ class SourceProgram:
     system_include_paths: tuple[str, ...] = tuple()
     flags: tuple[str, ...] = tuple()
 
+    def __post_init__(self) -> None:
+        """Sanity checks"""
+
+        # macros should not include "-D"
+        for macro in chain(self.defined_macros, self.available_macros):
+            assert not macro.strip().startswith("-D")
+
+        assert set(self.defined_macros) <= set(self.available_macros)
+
+        # (system) include paths must actually by paths and not include a flag
+        for path in self.include_paths:
+            assert not path.strip().startswith("-I")
+        for path in self.system_include_paths:
+            assert not path.strip().startswith("-isystem")
+
+        # XXX: can't check flags as the args and flags may be split
+
     def get_compilation_flags(self) -> tuple[str, ...]:
         """Returns flags based on the program's flags, include paths and macro defs.
 
@@ -270,6 +287,7 @@ class CompileError(Exception):
 
     @staticmethod
     def from_called_process_exception(
+        cmd: str,
         e: subprocess.CalledProcessError,
     ) -> CompileError:
         """Convertion from a CalledProcessError
@@ -283,30 +301,27 @@ class CompileError(Exception):
                 error containing the stdout and stderr of the compiler invocation
         """
 
-        output = ""
+        output = cmd
         if e.stdout:
             output += "\nSTDOUT====\n" + e.stdout.decode("utf-8")
         if e.stderr:
-            output = "\nSTDERR====\n" + e.stderr.decode("utf-8")
+            output += "\nSTDERR====\n" + e.stderr.decode("utf-8")
         return CompileError(output)
 
 
-class CompileContext:
-    def __init__(self, code: str):
-        self.code = code
+class TemporarySourceCodeFile:
+    def __init__(self, program: SourceProgram):
+        self.program = program
         self.fd_code: Optional[int] = None
-        self.fd_asm: Optional[int] = None
-        self.code_file: Optional[str] = None
-        self.asm_file: Optional[str] = None
-
-    def __enter__(self) -> tuple[str, str]:
-        self.fd_code, self.code_file = tempfile.mkstemp(suffix=".c")
-        self.fd_asm, self.asm_file = tempfile.mkstemp(suffix=".s")
+        self.code_file: str | None = None
 
+    def __enter__(self) -> Path:
+        self.fd_code, self.code_file = tempfile.mkstemp(
+            suffix=self.program.language.to_suffix()
+        )
         with open(self.code_file, "w") as f:
-            f.write(self.code)
-
-        return (self.code_file, self.asm_file)
+            f.write(self.program.code)
+        return Path(self.code_file).absolute()
 
     def __exit__(
         self,
@@ -314,14 +329,9 @@ class CompileContext:
         exc_value: Optional[BaseException],
         exc_traceback: Optional[TracebackType],
     ) -> None:
-        if self.code_file and self.fd_code and self.asm_file and self.fd_asm:
+        if self.code_file and self.fd_code:
             os.remove(self.code_file)
             os.close(self.fd_code)
-            # In case of a CompileError,
-            # the file itself might not exist.
-            if Path(self.asm_file).exists():
-                os.remove(self.asm_file)
-            os.close(self.fd_asm)
         else:
             raise CompileError("Compiler context exited but was not entered")
 
@@ -332,6 +342,35 @@ class CompilationInfo:
     stdout_stderr_output: str
 
 
+class CompilationOutputKind(Enum):
+    Object = 0
+    Assembly = 1
+    Exec = 2
+    Unspecified = 3
+
+    def to_flag(self) -> str:
+        match self:
+            case CompilationOutputKind.Object:
+                return "-c"
+            case CompilationOutputKind.Assembly:
+                return "-S"
+            case CompilationOutputKind.Exec | CompilationOutputKind.Unspecified:
+                return ""
+
+
+@dataclass(frozen=True)
+class CompilationOutput:
+    filename: Path
+    kind: CompilationOutputKind
+
+    def to_cmd(self) -> str:
+        if self.kind == CompilationOutputKind.Unspecified:
+            return ""
+        if kind_flag := self.kind.to_flag():
+            return kind_flag + " -o " + str(self.filename)
+        return "-o " + str(self.filename)
+
+
 @dataclass(frozen=True, kw_only=True)
 class CompilationSetting:
     """
@@ -343,11 +382,19 @@ class CompilationSetting:
         opt_level (OptLevel):
             which optimization level to use
         flags (tuple[str,...]):
-            which flags to use when compiling programs
-        include_path (tuple[str,...]):
-            which include paths to pass to the compiler (passed with -I)
+            which flags to use when compiling programs including "dashes" and
+            arguments, e.g., flags = ("-XX", "--foo", "bar")
+        include_paths (tuple[str,...]):
+            which include paths to pass to the compiler (passed to the compiler
+            with -I), e.g., include_paths=tuple("/path/1", "path/2",...)
         system_include_path (tuple[str,...]):
-            which system include paths to pass to the compiler (passed with -isystem)
+            which system include paths to pass to the compiler
+            (passed to the compiler with -isystem) with -I),
+            e.g., include_paths=tuple("/path/1", "path/2",...)
+        macro_definitions (tuple[str,...]):
+            which macro definition to pass to the compiler,
+            without the -D prefix, e.g., macros_defintions = tuple("MACRO1",
+            "MACROWITHARG=1", "MACROWITHARGUMENT=2")
     """
 
     compiler: CompilerExe
@@ -355,37 +402,98 @@ class CompilationSetting:
     flags: tuple[str, ...] = tuple()
     include_paths: tuple[str, ...] = tuple()
     system_include_paths: tuple[str, ...] = tuple()
+    macro_definitions: tuple[str, ...] = tuple()
     # TODO: timeout_s: int = 8
 
-    def get_compilation_base_cmd(self, program: SourceProgram) -> list[str]:
-        """Combines the program's flags with the flags of this compilation
-        setting.
+    def __post_init__(self) -> None:
+        """Sanity checks"""
+
+        # macros should not include "-D"
+        for macro in self.macro_definitions:
+            assert not macro.strip().startswith("-D")
+
+        # (system) include paths must actually by paths and not include a flag
+        for path in self.include_paths:
+            assert not path.strip().startswith("-I")
+        for path in self.system_include_paths:
+            assert not path.strip().startswith("-isystem")
+
+        # XXX: can't check flags as the args and flags may be split
+
+    def get_compilation_cmd(
+        self,
+        program: tuple[SourceProgram, Path],
+        output: CompilationOutput,
+        include_language_flags: bool = True,
+    ) -> list[str]:
+        """Assembles a compilation invocation based on the
+        input programs and the output.
 
         Args:
-            program(SourceProgram):
-                the program whose flags will be extracted
-
+            program (tuple[SourceProgram, Path]):
+                flags from the SourcePrograms are extracted and
+                the corresponding path are included in the output
+            output (CompilationOutput):
+                the output path and potentially additional flags
+                based on the output kind
+            include_language_flags (bool):
+                whether to include additional flags that specify
+                the source language and relevant linker flags
         Returns:
-            list[str]:
-                list of flags
+            str:
+                The assembled compilation command
         """
         cmd = list(
             chain(
                 (
                     str(self.compiler.exe),
-                    program.language.get_language_flag(),
                     f"-{self.opt_level.name}",
                 ),
+                (program[0].language.get_language_flag(),)
+                if include_language_flags
+                else ("",),
                 self.flags,
                 (f"-I{path}" for path in self.include_paths),
                 (f"-isystem{path}" for path in self.system_include_paths),
-                program.get_compilation_flags(),
+                (f"-D{macro}" for macro in self.macro_definitions),
+                program[0].get_compilation_flags(),
+                (str(program[1]),),
             )
         )
-        if linker_flag := program.language.get_linker_flag():
-            cmd.append(linker_flag)
+
+        if include_language_flags and output.kind == CompilationOutputKind.Exec:
+            if linker_flag := program[0].language.get_linker_flag():
+                cmd.append(linker_flag)
+        cmd.append(output.to_cmd())
         return cmd
 
+    def compile_program(
+        self,
+        program: SourceProgram,
+        output: CompilationOutput,
+        additional_flags: tuple[str, ...] = tuple(),
+        timeout: Optional[int] = None,
+    ) -> CompilationInfo:
+
+        with TemporarySourceCodeFile(program) as code_file:
+            cmd = self.get_compilation_cmd((program, code_file), output, True) + list(
+                additional_flags
+            )
+            try:
+                command_output = run_cmd(
+                    cmd,
+                    timeout=timeout,
+                    additional_env={"TMPDIR": str(tempfile.gettempdir())},
+                )
+            except subprocess.CalledProcessError as e:
+                raise CompileError.from_called_process_exception(" ".join(cmd), e)
+            return CompilationInfo(
+                source_file=Path(code_file),
+                stdout_stderr_output=command_output.stdout
+                + "\n"
+                + command_output.stderr,
+            )
+
     def get_llvm_ir_from_program(
         self, program: SourceProgram, timeout: Optional[int] = None
     ) -> str:
@@ -419,60 +527,16 @@ class CompilationSetting:
             str:
                 assembly code
         """
-        with CompileContext(program.code) as context_res:
-            code_file, asm_file = context_res
-            cmd = (
-                self.get_compilation_base_cmd(program)
-                + [
-                    code_file,
-                    "-S",
-                    "-o",
-                    asm_file,
-                ]
-                + list(additional_flags)
+        with tempfile.NamedTemporaryFile(suffix=".s") as asm_file:
+            self.compile_program(
+                program,
+                CompilationOutput(Path(asm_file.name), CompilationOutputKind.Assembly),
+                additional_flags,
+                timeout,
             )
-
-            try:
-                run_cmd(
-                    cmd,
-                    timeout=timeout,
-                    additional_env={"TMPDIR": str(tempfile.gettempdir())},
-                )
-            except subprocess.CalledProcessError as e:
-                raise CompileError.from_called_process_exception(e)
-
-            with open(asm_file, "r") as f:
+            with open(asm_file.name, "r") as f:
                 return f.read()
 
-    def _compile_program_to_X(
-        self,
-        program: SourceProgram,
-        output_file: Optional[Path],
-        flags: tuple[str, ...] = tuple(),
-        timeout: Optional[int] = None,
-    ) -> CompilationInfo:
-
-        with CompileContext(program.code) as context_res:
-            code_file, _ = context_res
-            cmd = (
-                self.get_compilation_base_cmd(program)
-                + [code_file]
-                + (["-o", str(output_file)] if output_file else [])
-                + list(flags)
-            )
-            try:
-                output = run_cmd(
-                    cmd,
-                    timeout=timeout,
-                    additional_env={"TMPDIR": str(tempfile.gettempdir())},
-                )
-            except subprocess.CalledProcessError as e:
-                raise CompileError.from_called_process_exception(e)
-            return CompilationInfo(
-                source_file=Path(code_file),
-                stdout_stderr_output=output.stdout + "\n" + output.stderr,
-            )
-
     def compile_program_to_object(
         self,
         program: SourceProgram,
@@ -493,8 +557,11 @@ class CompilationSetting:
                 information about the compilation
         """
 
-        return self._compile_program_to_X(
-            program, object_file, ("-c",) + additional_flags, timeout=timeout
+        return self.compile_program(
+            program,
+            CompilationOutput(object_file, CompilationOutputKind.Object),
+            additional_flags,
+            timeout=timeout,
         )
 
     def compile_program_to_executable(
@@ -518,8 +585,11 @@ class CompilationSetting:
         """
 
         # TODO: check/set file permissions of executable_path if it exists?
-        return self._compile_program_to_X(
-            program, executable_path, additional_flags, timeout=timeout
+        return self.compile_program(
+            program,
+            CompilationOutput(executable_path, CompilationOutputKind.Exec),
+            additional_flags,
+            timeout=timeout,
         )
 
     def preprocess_program(
@@ -542,8 +612,11 @@ class CompilationSetting:
 
         """
 
-        return self._compile_program_to_X(
-            program, None, ("-P", "-E") + additional_flags, timeout=timeout
+        return self.compile_program(
+            program,
+            CompilationOutput(Path(""), CompilationOutputKind.Unspecified),
+            ("-P", "-E") + additional_flags,
+            timeout=timeout,
         )
 
 
@@ -681,7 +754,7 @@ class ClangTool:
                     additional_env={"TMPDIR": str(tempfile.gettempdir())},
                 )
             except subprocess.CalledProcessError as e:
-                raise CompileError.from_called_process_exception(e)
+                raise CompileError.from_called_process_exception(" ".join(cmd), e)
 
             match mode:
                 case ClangToolMode.CAPTURE_OUT_ERR:
@@ -765,55 +838,56 @@ class CComp:
         return True
 
 
-def compiler_settings_parser() -> argparse.ArgumentParser:
+def __compilation_setting_parser() -> argparse.ArgumentParser:
     """Create an ArgumentParser for CompilationSetting.
     Should be integrated with another parser via the `parent`
     constructor argument as it does not create the `help` output
     itself (otherwise, it would not be composable anymore).
 
+    Note: -isystem is parsed as --isystem with a space after "m" as a
+    workaround, the input to the parser should be adjusted accordingly
+
     Args:
 
     Returns:
         argparse.ArgumentParser:
     """
     # We do not add the help hear so this parser can be extendend or extend another one.
-    parser = argparse.ArgumentParser(add_help=False)
+    parser = argparse.ArgumentParser(add_help=False, allow_abbrev=False)
 
     parser.add_argument(
         "compiler",
         type=Path,
     )
-    parser.add_argument("-I", action="append")
+    parser.add_argument("-I", type=str, action="append")
     parser.add_argument("-O")
-    parser.add_argument("-isystem", action="append")
+    parser.add_argument("--isystem", type=str, action="append")
     parser.add_argument("-D", action="append")
     parser.add_argument("-o")
-    # XXX: Parsed bug ignored for now (TM)
     parser.add_argument("-c", action="store_true")
     parser.add_argument("-S", action="store_true")
 
     return parser
 
 
-def parse_compile_settings(
-    parsed_args: argparse.Namespace, rest: list[str]
-) -> tuple[CompilationSetting, tuple[str, ...], list[str], str | None]:
-    """Parse a CompilationSetting from the provided parsed_args and rest.
-    The parser that yielded `parsed_args` and `rest` must include the
-    `compiler_settings_parser()` as a parent.
+def parse_compilation_setting_from_string(
+    s: str,
+) -> tuple[CompilationSetting, list[str], CompilationOutput]:
+    """Parse the compilation setting provided in `s`.
 
     Args:
-        parsed_args (argparse.Namespace): Argument namespace
-        rest (list[str]): Unparsed output
+        s (str): compilation command string to be parsed.
 
     Returns:
-        tuple[CompilationSetting, tuple[str, ...], list[str], str | None]:
+        tuple[CompilationSetting, tuple[str, ...], list[str], CompilationOutput]:
             - CompilationSetting
-            - macros set or unset
             - source files provided
-            - specified output file
+            - specified compilation output file and kind
     """
-
+    parser = __compilation_setting_parser()
+    parsed_args, rest = parser.parse_known_intermixed_args(
+        [p for p in s.replace("-isystem", "--isystem ").split()]
+    )
     CPP_EXT = (".cc", ".cxx", ".cpp")
     C_EXT = (".c",)
 
@@ -835,30 +909,28 @@ def parse_compile_settings(
 
     cexe = CompilerExe.from_path(Path(parsed_args.compiler))
 
+    opt = parsed_args.O
+    if not opt:
+        opt = "O0"
+
     csetting = CompilationSetting(
         compiler=cexe,
-        opt_level=OptLevel.from_str(parsed_args.O),
+        opt_level=OptLevel.from_str(opt),
         include_paths=include_paths,
         system_include_paths=system_include_paths,
+        macro_definitions=macro_definitions,
         flags=tuple(flags),
     )
-    return csetting, macro_definitions, sources, parsed_args.o
 
+    if parsed_args.o:
+        if parsed_args.c:
+            kind = CompilationOutputKind.Object
+        elif parsed_args.S:
+            kind = CompilationOutputKind.Assembly
+        else:
+            kind = CompilationOutputKind.Exec
+        output = CompilationOutput(parsed_args.o, kind)
+    else:
+        output = CompilationOutput(Path(""), CompilationOutputKind.Unspecified)
 
-def parse_compile_settings_from_string(
-    s: str,
-) -> tuple[CompilationSetting, tuple[str, ...], list[str], str | None]:
-    """Parse the compile settings provided in `s`.
-
-    Args:
-        s (str): compilation command string to be parsed.
-
-    Returns:
-        tuple[CompilationSetting, tuple[str, ...], list[str], str | None]:
-            see `parse_compile_settings`.
-    """
-    parser = argparse.ArgumentParser(parents=[compiler_settings_parser()])
-    parsed_args, rest = parser.parse_known_intermixed_args(
-        [p.strip() for p in s.split() if p]
-    )
-    return parse_compile_settings(parsed_args, rest)
+    return csetting, sources, output
